import typing
import types

from django.conf import settings

from acq import discovery

DEFAULT_MODULES_SETTING_NAME = 'INSTALLED_APPS'


def discover(
    *module_names: str,
    packages_setting_name: str = DEFAULT_MODULES_SETTING_NAME,
) -> typing.List[types.ModuleType]:
    """ Discover modules from packages listed in INSTALLED_APPS

    Provides a function to detect modules within INSTALLED_APPS or a given
    setting name. When provided, the setting with a name matching
    packages_setting_name will be searched instead of INSTALLED_APPS.

    TODO: Add support for AppConfig

    """

    packages = getattr(settings, packages_setting_name, [])
    return discovery.discover(*module_names, package_names=packages)



import importlib
import typing
import types


def import_module(module_name: str) -> typing.Optional[types.ModuleType]:
    """ Wrapper for importlib.import_module that ignores ImportError

    """

    try:
        return importlib.import_module(module_name)
    except ImportError:
        return None


def discover(
    *module_names: str,
    package_names: typing.Sequence[str] = [],
) -> typing.List[types.ModuleType]:
    """ Allows discovery of entire packages or specific modules within them """

    if not len(module_names):
        return discover(*package_names)

    if len(package_names):
        all_module_names = []

        for package_name in package_names:
            for module_name in module_names:
                all_module_names.append(package_name + '.' + module_name)

        return discover(*all_module_names)

    return list(filter(None, map(import_module, module_names)))
